@@ -4,14 +4,19 @@ from oauth2p.exceptions import UnsupportedGrantType, InvalidClient, InvalidGrant
 
 class Authorization(object):
 
-    def __init__(self, storage, token_length=40, expire=3600):
+    def __init__(self, storage, token_length=40, expire=3600, refresh_expire=3600):
         self.__storage = storage
         self.__token_length = token_length
         self.__expire = expire
+        self.__refresh_expire = refresh_expire
 
     def get_access_token(self, grant_type, *args, **kwargs):
         return self.token_handlers.get(grant_type, self.__default_token_handler)(self, *args, **kwargs)
 
+    @property
+    def refresh_expire(self):
+        return self.__refresh_expire
+
     @property
     def token_handlers(self):
         return self.__grant_type_handlers
@@ -49,26 +54,48 @@ class Authorization(object):
         }
 
     def __password(self, **kwargs):
+        """Returns response if valid credentials:
+            {
+                access_token: <access_token>
+                token_type: <token_type>
+                expires_in: <expires_in> (can be None if persistent)
+                refresh_token: <refresh_token> (optional)
+            }
+        Passes all other keyword arguments to the storage save method and storage is_valid_owner methods
+
+        Keyword arguments:
+        username -- username
+        password -- password
+        client_id -- client_id (optional)
+        return_refresh -- boolean if should return refresh
+
+        """
+        response = {}
         username = kwargs.pop('username', None)
         password = kwargs.pop('password', None)
+        client_id = kwargs.pop('client_id', None)
+        return_refresh = kwargs.pop('return_refresh', None)
 
         if not username or password is None or not self.storage.is_valid_owner(username, password, **kwargs):
             raise InvalidClient()
 
         token = utils.generate_random_vschar(self.token_length)
-        refresh_token = utils.generate_random_vschar(self.token_length)
         token_data = kwargs
         token_data['username'] = username
-        token_key = self.storage.save(token=token, token_type='access_token',
-                                      client_id=None, data=token_data, expire=self.expire)
-        refresh_key = self.storage.save(token=refresh_token, token_type='refresh_token',
-                                        client_id=None, data=token_data, expire=None)
-        return {
-            "access_token": token,
-            "token_type": self.token_type,
-            "expires_in": self.expire,
-            "refresh_token": refresh_token
-        }
+        self.storage.save(token=token, token_type='access_token',
+                          client_id=client_id, data=token_data, expire=self.expire)
+
+        response['access_token'] = token
+        response['token_type'] = self.token_type
+        response['expires_in'] = self.expire
+
+        if return_refresh:
+            refresh_token = utils.generate_random_vschar(self.token_length)
+            self.storage.save(token=refresh_token, token_type='refresh_token',
+                              client_id=client_id, data=token_data, expire=self.refresh_expire)
+            response['refresh_token'] = refresh_token
+
+        return response
 
     def __refresh_token(self, **kwargs):
         client_id = kwargs.get('client_id', None)
